from dataclasses import dataclass

from apexdevkit.formatter import DataclassFormatter
from apexdevkit.http import FakeHttp, FluentHttp, HttpMethod
from apexdevkit.http.fake import FakeResponse
from faker import Faker

from apexauthlib.integration import AuthApiProvider
from apexauthlib.integration.api import AuthApi


@dataclass(frozen=True)
class TestPermissions:
    role: str
    read: bool
    write: bool


def test_login() -> None:
    http = FakeHttp(FakeResponse({"access_token": "token"}))
    service_name = Faker().text()
    api = AuthApiProvider[TestPermissions](
        FluentHttp(http), service_name, DataclassFormatter(TestPermissions)
    )

    result = api.login("user", "9")

    assert result == "token"

    http.intercepted(HttpMethod.post).on_endpoint("/auth/login")

    assert http.data == {
        "grant_type": "password",
        "username": "user",
        "password": "9",
        "scope": "",
        "client_id": "string",
        "client_secret": "string",
    }


def test_user() -> None:
    http = FakeHttp(FakeResponse({"id": "1"}))
    service_name = Faker().text()
    api = AuthApi[TestPermissions](
        FluentHttp(http), service_name, DataclassFormatter(TestPermissions), "token"
    )

    result = api.user()

    assert result == "1"

    http.intercepted(HttpMethod.get).on_endpoint("/auth/user")

    assert http.headers == {
        "Authorization": "Bearer token",
    }


def test_metadata() -> None:
    http = FakeHttp(
        FakeResponse(
            {
                "data": {
                    "metadata": {
                        "metadata": {"role": "admin", "read": True, "write": True}
                    }
                }
            }
        )
    )

    service_name = Faker().text()
    api = AuthApi[TestPermissions](
        FluentHttp(http), service_name, DataclassFormatter(TestPermissions), "token"
    )

    result = api.metadata_for("1")

    assert result == TestPermissions("admin", True, True)

    http.intercepted(HttpMethod.get).on_endpoint(f"/services/{service_name}/metadata/1")

    assert http.headers == {
        "Authorization": "Bearer token",
    }
